<?php

function signature($secretId, $secretKey, $method='GET', $path='/', $params=array(), $headers=array(), $expire=120) {
    $filter_headers = array();
    foreach ($headers as $k => $v) {
        $lower_key = strtolower($k);
        if ($lower_key == 'content-type' || $lower_key == 'content-md5' || $lower_key == 'host' || $lower_key[0] == 'x') {
            $filter_headers[$lower_key] = $v;
        }
    }
    $filter_params = array();
    foreach ($params as $k => $v) {
        $filter_params[strtolower($k)] = $v;
    }
    ksort($filter_params);
    ksort($filter_headers);
    $filter_headers = array_map('strtolower', $filter_headers);
    $uri_headers = http_build_query($filter_headers);
    $httpString = strtolower($method) . "\n" . urldecode($path) .
            "\n". http_build_query($filter_params) . "\n" . $uri_headers . "\n";
    $signTime = (string)(time() - 60) . ';' . (string)(time() + $expire);
    #$signTime = "1510109254;1510109314";
    $stringToSign = "sha1\n" . $signTime . "\n" . sha1($httpString) . "\n";
    $signKey = hash_hmac('sha1', $signTime, $secretKey);
    $signature = hash_hmac('sha1', $stringToSign, $signKey);
    return "q-sign-algorithm=sha1&q-ak=$secretId" .
            "&q-sign-time=$signTime&q-key-time=$signTime&q-header-list=" .
            join(";", array_keys($filter_headers)) . "&q-url-param-list=" .
            join(";", array_keys($filter_params)) . "&q-signature=$signature";
}

// Test Case
echo signature('AKIDc9YlmrBcFk4C8sbmXQ8i65XXXXXXXXXX', 'LUSE4nPK1d4tX5SHyXv6tZXXXXXXXXXX',
    'GET', '/logset',
    array('logset_id' => 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx'),
    array('Host' => 'ap-shanghai.cls.myqcloud.com', 'User-Agent' => 'AuthSDK'), 300) . "\n";

echo signature('AKIDc9YlmrBcFk4C8sbmXQ8i65XXXXXXXXXX', 'LUSE4nPK1d4tX5SHyXv6tZXXXXXXXXXX',
    'PUT', '/logset',
    array(),
    array('Host' => 'ap-shanghai.cls.myqcloud.com',
          'Content-Type' => 'application/json',
          'Content-MD5' => 'f9c7fc33c7eab68dfa8a52508d1f4659',
          'User-Agent' => 'AuthSDK'), 300) . "\n";